@@ -1,6 +1,6 @@
 package HTML::FormFu::FakeQuery;
 use strict;
-use Carp qw/ croak /;
+use Carp qw( croak );
 
 sub new {
     my ( $class, $form, $param ) = @_;
@@ -10,8 +10,9 @@ sub new {
 
     # handle pre-expanded input
 
-    my @names = grep {defined}
-        map { $_->nested_name } @{ $form->get_fields };
+    my @names
+        = grep {defined}
+        map    { $_->nested_name } @{ $form->get_fields };
 
     for my $name (@names) {
         next if exists $param->{$name};
@@ -41,9 +42,10 @@ sub param {
     elsif ( @_ == 1 ) {
         my ($param) = @_;
 
-        unless ( exists $self->{_params}{$param} ) {
+        if ( !exists $self->{_params}{$param} ) {
             return wantarray ? () : undef;
         }
+
         if ( ref $self->{_params}{$param} eq 'ARRAY' ) {
             return (wantarray)
                 ? @{ $self->{_params}{$param} }
